package iobuffers;

import java.io.ByteArrayOutputStream;
import java.io.DataOutput;
import java.io.DataOutputStream;
import java.io.IOException;


/**
 * This class implements an output stream in which the data is written
 * into a buffer and provides methods to write primitive data types.
 * The buffer automatically grows as data is written to it.
 * It extends a <code>ByteArrayOutputStream</code> and relies on a
 * <code>DataOutputStream</code> to write primitive data types.
 */
public class OutputBuffer extends ByteArrayOutputStream implements DataOutput {

    /** DataOutputStream used to write primitive data types. */
    private final DataOutputStream out;


    /**
     * Creates a new <code>OutputBuffer</code> with a buffer capacity
     * of the default size (as the class <code>ByteArrayOutputStream</code>),
     * though its size increases if necessary.
     */
    public OutputBuffer() {
        super();
        out = new DataOutputStream(this);
    }


    /**
     * Creates a new <code>OutputBuffer</code>, with a
     * buffer capacity of the specified size, in bytes.
     * @param size the initial size.
     */
    public OutputBuffer(int size) {
        super(size);
        out = new DataOutputStream(this);
    }


    /**
     * Gets a reference to the buffer where data is stored.
     * Use with caution: if possible use <code>toByteArray</code> instead.
     * @return the buffer where data is stored.
     */
    public synchronized byte[] buffer() {
        return this.buf;
    }
    
    
    @Override
    public void write(byte[] b) {
        this.write(b, 0, b.length);
    }
    

    public void writeBoolean(boolean v) {
        try {
            out.writeBoolean(v);
        } catch (IOException ex) { } // no exception will be thrown
    }


    public void writeByte(int v) {
        try {
            out.writeByte(v);
        } catch (IOException ex) { } // no exception will be thrown
    }


    public void writeShort(int v) {
        try {
            out.writeShort(v);
        } catch (IOException ex) { } // no exception will be thrown
    }


    public void writeChar(int v) {
        try {
            out.writeChar(v);
        } catch (IOException ex) { } // no exception will be thrown
    }


    public void writeInt(int v) {
        try {
            out.writeInt(v);
        } catch (IOException ex) { } // no exception will be thrown
    }


    public void writeLong(long v) {
        try {
            out.writeLong(v);
        } catch (IOException ex) { } // no exception will be thrown
    }


    public void writeFloat(float v) {
        try {
            out.writeFloat(v);
        } catch (IOException ex) { } // no exception will be thrown
    }


    public void writeDouble(double v) {
        try {
            out.writeDouble(v);
        } catch (IOException ex) { } // no exception will be thrown
    }


    public void writeBytes(String s) {
        try {
            out.writeBytes(s);
        } catch (IOException ex) { } // no exception will be thrown
    }


    public void writeChars(String s) {
        try {
            out.writeChars(s);
        } catch (IOException ex) { } // no exception will be thrown
    }


    public void writeUTF(String s) {
        try {
            out.writeUTF(s);
        } catch (IOException ex) { } // no exception will be thrown
    }
    
}
